//
//  DocumentWindowController.m
//  Company
//
//  Created by Amin Negm-Awad on So.,16.09.12.
//  Copyright (c) 2012 Amin Negm-Awad. All rights reserved.
//

#import "DocumentWindowController.h"

NSString* const personDeletionNotification
= @"com.cocoading.company.documentWC.notifications.personDeletion";

NSString* const personsPlaceholderItem
= @"com.cocoading.company.documentWC.PersonsPlaceholder";

@interface DocumentWindowController ()

@end

@implementation DocumentWindowController
- (IBAction)removePerson:(id)sender
{
	NSNotificationCenter* center = [NSNotificationCenter defaultCenter];
	// Notification
	NSArray *selectedObjects = [self.personsController selectedObjects];
	for (NSManagedObject *person in selectedObjects) {
		[center postNotificationName:personDeletionNotification object:person];
	}
	// Remove
	[self.personsController removeObjects:selectedObjects];
}

#pragma mark -
#pragma mark Actions
- (IBAction)addGroup:(id)sender
{
	NSManagedObjectContext *context = [[self document] managedObjectContext];
	NSEntityDescription *entity = [NSEntityDescription entityForName:@"Group"
															inManagedObjectContext:context];
	NSManagedObject *group = [[NSManagedObject alloc] initWithEntity:entity
												 insertIntoManagedObjectContext:context];
	[self.groupsController addObject:group];
	[self.sidebarView reloadData];
	
	NSInteger row = [self.sidebarView rowForItem:group];
	[self.sidebarView editColumn:0 row:row withEvent:nil select:YES];
	self.selectedGroup = group;
}

- (IBAction)removeGroup:(id)sender
{
	[self.groupsController removeObject:self.selectedGroup];
	[self.sidebarView reloadData];
	
	NSInteger row = [self.sidebarView selectedRow];
	if (row != -1) {
		[self.sidebarView deselectRow:row];
		self.selectedGroup = nil;
	}
}
#pragma mark -
#pragma mark Data Source
- (NSInteger)           outlineView:(NSOutlineView*)outlineView
	          numberOfChildrenOfItem:(id)item
{
	if( !item ) {
		return [[self.groupsController arrangedObjects] count]+ 1;
	}
	return 0;
}

- (id)outlineView:(NSOutlineView*)outlineView
				child:(NSInteger)index ofItem:(id)item
{
	if( !item ) {
		if( index == 0 ) {
			return personsPlaceholderItem;
		} else {
			return [[self.groupsController arrangedObjects] objectAtIndex:index-1];
		}
	}
	return nil;
}

- (BOOL)     outlineView:(NSOutlineView*)outlineView
        isItemExpandable:(id)item
{
	return NO;
}

- (id) outlineView:(NSOutlineView*)outlineView
objectValueForTableColumn:(NSTableColumn*)tableColumn byItem:(id)item
{
	if( item == personsPlaceholderItem ) {
		return @"Persons";
	}
	return [item valueForKey:@"name"];
}

- (void) outlineView:(NSOutlineView*)outlineView
		setObjectValue:(id)value forTableColumn:(NSTableColumn*)tableColumn
				  byItem:(id)item
{
	[item setValue:value forKey:@"name"];
}
#pragma mark -
#pragma mark Delegate
- (BOOL)outlineView:(NSOutlineView*)outlineView shouldSelectItem:(id)item
{
	if (item==personsPlaceholderItem) {
		return NO;
	}
	
	NSString* entityName = [[item entity] name];
	if ([entityName isEqualToString:@"Group"]) {
		self.selectedGroup = item;
	} else {
		self.selectedGroup = nil;
	}
	return YES;
}
- (BOOL)outlineView:(NSOutlineView*)outlineView
		  isGroupItem:(id)item
{
	if( item == personsPlaceholderItem ) {
		return YES;
	}
	return NO;
}
#pragma mark -
#pragma mark Creation
- (void)awakeFromNib
{
	NSOutlineView *sidebar = self.sidebarView;
	[sidebar performSelector:@selector(reloadData) withObject:nil afterDelay:0];
}

- (id)initWithWindowNibName:(NSString*)nibName
{
    self = [super initWithWindowNibName:nibName];
    if (self) {
    }
    
    return self;
}
@end
