//
//  DocumentWindowController.m
//  Company
//
//  Created by Amin Negm-Awad on So.,16.09.12.
//  Copyright (c) 2012 Amin Negm-Awad. All rights reserved.
//

#import "DocumentWindowController.h"

NSString* const personDeletionNotification
= @"com.cocoading.company.documentWC.notifications.personDeletion";

NSString* const personsPlaceholderItem
= @"com.cocoading.company.documentWC.PersonsPlaceholder";

@interface DocumentWindowController ()

@end

@implementation DocumentWindowController
#pragma mark Panes
- (void)selectPane:(NSString*)nibName
{
	// Lade den View-Controller
	NSViewController *newController
	= [[NSViewController alloc] initWithNibName:nibName bundle:nil];
	[newController setRepresentedObject:self];
	NSView *newView = [newController view];
	
	// Übernehme den Frame und Autolayout
	NSView *oldView = self.displayedView;
	NSRect frame = [oldView frame];
	[newView setFrame:frame];
	
	// Austauschen
	NSView * superview = [oldView superview];
	[oldView removeFromSuperview];
	[superview addSubview:newView];
	
	NSLayoutConstraint *constraint;
	constraint = [NSLayoutConstraint constraintWithItem:newView
															attribute:NSLayoutAttributeTop
															relatedBy:NSLayoutRelationEqual
																toItem:superview
															attribute:NSLayoutAttributeTop
														  multiplier:1 constant:0];
	[superview addConstraint:constraint];
	
	constraint = [NSLayoutConstraint constraintWithItem:newView
															attribute:NSLayoutAttributeBottom
															relatedBy:NSLayoutRelationEqual
																toItem:superview attribute:NSLayoutAttributeBottom
														  multiplier:1 constant:0];
	[superview addConstraint:constraint];
	
	constraint = [NSLayoutConstraint constraintWithItem:newView
															attribute:NSLayoutAttributeRight
															relatedBy:NSLayoutRelationEqual
																toItem:superview
															attribute:NSLayoutAttributeRight
														  multiplier:1 constant:0];
	[superview addConstraint:constraint];
	
	constraint = [NSLayoutConstraint constraintWithItem:newView
															attribute:NSLayoutAttributeLeft
															relatedBy:NSLayoutRelationEqual
																toItem:self.sidebarView
															attribute:NSLayoutAttributeRight
														  multiplier:1 constant:0];
	[superview addConstraint:constraint];
	
	// Merke neuen View und View-Controller
	self.viewController = newController;
	self.displayedView = newView;
}
#pragma mark -
#pragma mark Actions
- (IBAction)addGroup:(id)sender
{
	NSManagedObjectContext *context = [[self document] managedObjectContext];
	NSEntityDescription *entity = [NSEntityDescription entityForName:@"Group"
															inManagedObjectContext:context];
	NSManagedObject *group = [[NSManagedObject alloc] initWithEntity:entity
												 insertIntoManagedObjectContext:context];
	[self.groupsController addObject:group];
	[self.sidebarView reloadData];
	
	NSInteger row = [self.sidebarView rowForItem:group];
	[self.sidebarView editColumn:0 row:row withEvent:nil select:YES];
	self.selectedGroup = group;
	self.selectedPerson = nil;
	[self selectPane:@"GroupPane"];
}

- (IBAction)removeGroup:(id)sender
{
	[self.groupsController removeObject:self.selectedGroup];
	[self.sidebarView reloadData];
	
	NSInteger row = [self.sidebarView selectedRow];
	if (row != -1) {
		[self.sidebarView deselectRow:row];
		self.selectedGroup = nil;
	}
}

- (IBAction)addPerson:(id)sender
{
	NSManagedObjectContext *context = [self.document managedObjectContext];
	NSEntityDescription *entity = [NSEntityDescription entityForName:@"Person"
															inManagedObjectContext:context];
	NSManagedObject *person = [[NSManagedObject alloc] initWithEntity:entity
												  insertIntoManagedObjectContext:context];
	[self.personsController addObject:person];
	[self.sidebarView reloadData];
	[self.sidebarView expandItem:personsPlaceholderItem];
	
	NSInteger row = [self.sidebarView rowForItem:person];
	NSIndexSet *rows = [NSIndexSet indexSetWithIndex:row];
	[self.sidebarView selectRowIndexes:rows byExtendingSelection:NO];
	self.selectedGroup = nil;
	self.selectedPerson = person;
	[self selectPane:@"PersonPane"];
}

- (IBAction)removePerson:(id)sender
{
	[self.personsController remove:sender];
	self.selectedPerson = nil;
	[self.sidebarView reloadData];
}

- (IBAction)remove:(id)sender
{
	id item = [self.sidebarView itemAtRow:[self.sidebarView selectedRow]];
	[self.window endEditingFor:item];
	if( [[[item entity] name] isEqualToString:@"Person"] ) {
		[self removePerson:sender];
	}
	if( [[[item entity] name] isEqualToString:@"Group"] ) {
		[self removeGroup:sender];
	}
	[self selectPane:@"EmptyPane"];
	[self.sidebarView selectRowIndexes:[NSIndexSet indexSet]
					  byExtendingSelection:NO];
}
- (IBAction)updatePerson:(id)sender
{
	NSManagedObject *person = self.selectedPerson;
	if( [[[person entity] name] isEqualToString:@"Person"] ) {
		[self.sidebarView reloadItem:person];
	}
}
#pragma mark -
#pragma mark Data Source
- (NSInteger)           outlineView:(NSOutlineView*)outlineView
	          numberOfChildrenOfItem:(id)item
{
	if( !item ) {
		return [[self.groupsController arrangedObjects] count]+ 1;
	}
	
	if( item == personsPlaceholderItem ) {
		return [[self.personsController arrangedObjects] count];
	}
	
	return 0;
}

- (id)outlineView:(NSOutlineView*)outlineView
				child:(NSInteger)index ofItem:(id)item
{
	if( !item ) {
		if( index == 0 ) {
			return personsPlaceholderItem;
		} else {
			return [[self.groupsController arrangedObjects] objectAtIndex:index-1];
		}
	}
	
	if( item == personsPlaceholderItem ) {
		return [[self.personsController arrangedObjects] objectAtIndex:index];
	}
	
	return nil;
}

- (BOOL)     outlineView:(NSOutlineView*)outlineView
        isItemExpandable:(id)item
{
	if( item == personsPlaceholderItem ) {
		return YES;
	} else {
		return NO;
	}
}

- (id) outlineView:(NSOutlineView*)outlineView
objectValueForTableColumn:(NSTableColumn*)tableColumn byItem:(id)item
{
	if( item == personsPlaceholderItem ) {
		return @"Persons";
	}
	
	if( [[[item entity] name] isEqualToString:@"Person"] ) {
		return [NSString stringWithFormat:@"%@, %@",
				  [item valueForKey:@"lastName"],
				  [item valueForKey:@"firstName"]];
	}
	
	return [item valueForKey:@"name"];
}

- (void) outlineView:(NSOutlineView*)outlineView
		setObjectValue:(id)value forTableColumn:(NSTableColumn*)tableColumn
				  byItem:(id)item
{
	[item setValue:value forKey:@"name"];
}
#pragma mark -
#pragma mark Dragging
- (BOOL)outlineView:(NSOutlineView*)outlineView
			writeItems:(NSArray*)items toPasteboard:(NSPasteboard *)pasteboard
{
	// Hole die Daten
	NSMutableArray *pbItems = [NSMutableArray arrayWithCapacity:[items count]];
	for( id item in items ) {
		// Haben wir ein managed Object?
		if( ![item isKindOfClass:[NSManagedObject class]] ) {
			return NO;
		}
		
		NSManagedObject *person = item;
		if( ![[[person entity] name] isEqualToString:@"Person"] ) {
			continue;
		}
		
		NSManagedObjectID *personID = [person objectID];
		NSURL *objectURI = [personID URIRepresentation];
		[pbItems addObject:objectURI];
	}
		
	if( [pbItems count] == 0 ) {
		return NO;
	}
	return [pasteboard writeObjects:pbItems];
}

- (NSDragOperation)outlineView:(NSOutlineView *)outlineView
						validateDrop:(id<NSDraggingInfo>)info
						proposedItem:(id)destinationItem
				proposedChildIndex:(NSInteger)index
{
	// ist das Ziel ein Managed-Object?
	if( ![destinationItem isKindOfClass:[NSManagedObject class]] ) {
		return NSDragOperationNone;
	}
	NSManagedObject *destinationObject = destinationItem;
	
	// Ist das Ziel eine Gruppe?
	if( ![[[destinationObject entity] name] isEqualToString:@"Group"] ) {
		return NSDragOperationNone;
	}
	
	// Hole das Pasteboard
	NSPasteboard* pboard = [info draggingPasteboard];
	
	// Hole die Daten
	NSArray *classes = [NSArray arrayWithObject:[NSURL class]];
	NSArray *pbItems = [pboard readObjectsForClasses:classes options:nil];
	
	// Wandele diese in Managed-Objects um
	NSManagedObjectContext *context = [self.document managedObjectContext];
	NSPersistentStoreCoordinator *coordinator = context.
	persistentStoreCoordinator;
	NSManagedObjectID *objectID;
	for( NSURL *objectURI in pbItems ) {
		objectID = [coordinator managedObjectIDForURIRepresentation:objectURI];
		if( objectID == nil ) {
			return NSDragOperationNone;
		}
	
		if( ![[[objectID entity] name] isEqualToString:@"Person"] ) {
			return NSDragOperationNone;
		}
	}
	return NSDragOperationCopy;
}

- (BOOL)outlineView:(NSOutlineView *)outlineView
			acceptDrop:(id<NSDraggingInfo>)info
					item:(id)destinationItem
			childIndex:(NSInteger)index
{
	// Hole das Pasteboard
	NSPasteboard* pboard = [info draggingPasteboard];
	
	// Hole die Daten
	NSArray *classes = [NSArray arrayWithObject:[NSURL class]];
	NSArray *pbItems = [pboard readObjectsForClasses:classes options:nil];
	
	// Wandele diese in Managed-Objects um
	NSManagedObjectContext *context = [self.document managedObjectContext];
	NSPersistentStoreCoordinator *coordinator
	= context.persistentStoreCoordinator;
	NSManagedObjectID *objectID;
	NSManagedObject *person;
	for( NSURL *objectURI in pbItems ) {
		objectID = [coordinator managedObjectIDForURIRepresentation:objectURI];
		person = [context objectWithID:objectID];
		[person setValue:destinationItem forKey:@"Group"];
	}
	return YES;
}
	
	
	
	
	
	
	
#pragma mark Delegate
- (BOOL)outlineView:(NSOutlineView*)outlineView shouldSelectItem:(id)item
{
	if (item==personsPlaceholderItem) {
		return NO;
	}
	
	NSString* entityName = [[item entity] name];
	if ([entityName isEqualToString:@"Group"]) {
		self.selectedGroup = item;
		self.selectedPerson = nil;
		[self selectPane:@"GroupPane"];
	} else if ([entityName isEqualToString:@"Person"]) {
		self.selectedPerson = item;
		self.selectedGroup = nil;
		[self selectPane:@"PersonPane"];
	} else {
		self.selectedPerson = nil;
		self.selectedGroup = nil;
		[self selectPane:@"EmptyPane"];
	}
	return YES;
}
- (BOOL)outlineView:(NSOutlineView*)outlineView
		  isGroupItem:(id)item
{
	if( item == personsPlaceholderItem ) {
		return YES;
	}
	return NO;
}
- (BOOL) outlineView:(NSOutlineView*)outlineView
shouldEditTableColumn:(NSTableColumn*)tableColumn
					 item:(id)item

{
	if( item == personsPlaceholderItem ) {
		return NO;
	}
	if( [[[item entity] name] isEqualToString:@"Person"] ) {
		return NO;
	}
	return YES;
}
#pragma mark -
#pragma mark Creation
- (void)awakeFromNib
{
	[self selectPane:@"EmptyPane"];
	NSOutlineView *sidebar = self.sidebarView;
	[sidebar performSelector:@selector(reloadData) withObject:nil afterDelay:0];
	
	NSArray* types = [NSArray arrayWithObject:@"public.url"];
	[self.sidebarView registerForDraggedTypes:types];
}

- (id)initWithWindowNibName:(NSString*)nibName
{
    self = [super initWithWindowNibName:nibName];
    if (self) {
    }
    
    return self;
}
@end
